<?php
/*
Plugin Name:    Naver IndexNow Single
Plugin URI:     https://www.thewordcracker.com/
Description:    Automates the submission of URLs to Naver IndexNow upon post publication, with correct handling of API key changes and immediate generation of the necessary API key file.
Version:        1.1.0
Author:         WordCracker
Author URI:     https://www.thewordcracker.com/
License:        GPL-2.0+
*/

if (!defined('ABSPATH')) exit;

date_default_timezone_set('Asia/Seoul');

function naver_indexnow_register_menu_page() {
    add_menu_page('Naver IndexNow Settings', 'Naver IndexNow', 'manage_options', 'naver-indexnow', 'naver_indexnow_menu_page', 'dashicons-admin-site');
}

function naver_indexnow_menu_page() {
    ?>
    <div class="wrap">
        <h2>Naver IndexNow Settings</h2>
        <form method="post" action="options.php">
            <?php
            settings_fields('naver_indexnow_option_group');
            do_settings_sections('naver-indexnow');
            submit_button('Save Changes');
            ?>
        </form>
        <?php if (isset($_GET['settings-updated']) && $_GET['settings-updated']) : ?>
            <div id="message" class="updated notice is-dismissible">
                <p><strong>API key has been successfully saved.</strong></p>
            </div>
        <?php endif; ?>
        <form method="post">
            <?php submit_button('Clear Logs', 'delete', 'clear_logs'); ?>
            <?php wp_nonce_field('naver_indexnow_clear_logs_action', 'naver_indexnow_clear_logs_nonce'); ?>
        </form>
        <?php
        if (isset($_POST['clear_logs']) && check_admin_referer('naver_indexnow_clear_logs_action', 'naver_indexnow_clear_logs_nonce')) {
            if (current_user_can('manage_options')) {
                naver_indexnow_clear_logs();
                echo '<div><p>Logs cleared successfully.</p></div>';
            } else {
                echo '<div><p>You do not have sufficient permissions to clear logs.</p></div>';
            }
        }
        naver_indexnow_display_logs();
        ?>
    </div>
    <?php
}

function naver_indexnow_admin_init() {
    register_setting(
        'naver_indexnow_option_group',
        'naver_indexnow_api_key',
        'naver_indexnow_sanitize_api_key'
    );

    add_settings_section(
        'naver_indexnow_api_key_section',
        'API Key Configuration',
        'naver_indexnow_section_info',
        'naver-indexnow'
    );

    add_settings_field(
        'naver_indexnow_api_key_field',
        'Naver API Key',
        'naver_indexnow_api_key_callback',
        'naver-indexnow',
        'naver_indexnow_api_key_section'
    );
}

function naver_indexnow_sanitize_api_key($new_value) {
    $old_value = get_option('naver_indexnow_api_key');
    if ($new_value != $old_value) {
        naver_indexnow_update_api_key_file($old_value, $new_value);
    }
    return sanitize_text_field($new_value);
}

function naver_indexnow_section_info() {
    echo 'Enter your Naver IndexNow API key and ensure the corresponding .txt file is placed in your root directory.';
}

function naver_indexnow_api_key_callback() {
    $api_key = get_option('naver_indexnow_api_key');
    echo '<input type="text" id="naver_indexnow_api_key" name="naver_indexnow_api_key" value="' . esc_attr($api_key) . '" />';
}

function naver_indexnow_update_api_key_file($old_value, $new_value) {
    if ($new_value != $old_value) {
        // 이전 API 키 파일 삭제
        if (!empty($old_value)) {
            $old_file_path = ABSPATH . sanitize_file_name($old_value) . '.txt';
            if (file_exists($old_file_path)) {
                unlink($old_file_path);
            }
        }

        // 새로운 API 키 파일 생성
        if (!empty($new_value)) {
            $new_file_path = ABSPATH . sanitize_file_name($new_value) . '.txt';
            if (false === file_put_contents($new_file_path, sanitize_text_field($new_value))) {
                add_settings_error('naver_indexnow_api_key', 'file_write_error', 'Failed to write API key file.', 'error');
            }
        }
    }
}

function naver_indexnow_submit_url($post_id) {
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) 
        return;

    if ('post' !== get_post_type($post_id)) 
        return;

    if (!current_user_can('edit_post', $post_id)) 
        return;

    $last_submitted_time = get_post_meta($post_id, '_indexnow_last_submitted', true);

    if ($last_submitted_time && (time() - $last_submitted_time) < 60) {
        return;
    }

    $post_url = get_permalink($post_id);
    $api_key = trim(get_option('naver_indexnow_api_key'));
    $key_location = home_url('/') . sanitize_file_name($api_key) . '.txt';
    $api_url = 'https://searchadvisor.naver.com/indexnow?url=' . urlencode($post_url) . '&key=' . $api_key . '&keyLocation=' . urlencode($key_location);

    $response = wp_remote_get($api_url);

    if (is_wp_error($response) || 200 != wp_remote_retrieve_response_code($response)) {
        $error_message = is_wp_error($response) ? $response->get_error_message() : 'HTTP error ' . wp_remote_retrieve_response_code($response);
        naver_indexnow_log($post_url, 0, 'Failure: ' . $error_message);
        return;
    }

    $http_code = wp_remote_retrieve_response_code($response);
    $response_body = wp_remote_retrieve_body($response);
    naver_indexnow_log($post_url, $http_code, 'Success: ' . $response_body);

    update_post_meta($post_id, '_indexnow_last_submitted', time());
}

add_action('save_post', 'naver_indexnow_submit_url', 10, 1);
add_action('bulk_edit_posts', 'naver_indexnow_bulk_edit_submit_url', 10, 1);

function naver_indexnow_bulk_edit_submit_url($post_ids) {
    foreach ($post_ids as $post_id) {
        naver_indexnow_submit_url($post_id);
    }
}

function naver_indexnow_log($url, $code, $message) {
    global $wpdb;
    $wpdb->insert(
        $wpdb->prefix . 'naver_indexnow_logs',
        array(
            'url' => $url,
            'code' => $code,
            'message' => $message,
            'created_at' => current_time('mysql', 1),
        ),
        array('%s', '%d', '%s', '%s')
    );
}

function naver_indexnow_display_logs() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'naver_indexnow_logs';
    $logs = $wpdb->get_results("SELECT * FROM $table_name ORDER BY created_at DESC", ARRAY_A);
    echo '<h3>Logs</h3>';
    echo '<table class="widefat fixed" cellspacing="0"><thead><tr><th>Timestamp</th><th>URL</th><th>Code</th><th>Message</th></tr></thead><tbody>';
    foreach ($logs as $log) {
        $local_time = new DateTime($log['created_at'], new DateTimeZone('UTC'));
        $timezone = get_option('timezone_string');
        if ($timezone) {
            $local_time->setTimezone(new DateTimeZone($timezone));
        } else {
            $local_time->setTimezone(new DateTimeZone('Asia/Seoul'));
        }
        echo '<tr><td>' . esc_html($local_time->format('Y-m-d H:i:s')) . '</td><td>' . esc_html($log['url']) . '</td><td>' . esc_html($log['code']) . '</td><td>' . esc_html($log['message']) . '</td></tr>';
    }
    echo '</tbody></table>';
}

function naver_indexnow_clear_logs() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'naver_indexnow_logs';
    $wpdb->query("DELETE FROM $table_name");
}

add_action('admin_menu', 'naver_indexnow_register_menu_page');
add_action('admin_init', 'naver_indexnow_admin_init');

register_activation_hook(__FILE__, 'naver_indexnow_create_log_table');

function naver_indexnow_create_log_table() {
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    global $wpdb;
    $table_name = $wpdb->prefix . 'naver_indexnow_logs';
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        url varchar(255) NOT NULL,
        code smallint(5) NOT NULL,
        message text NOT NULL,
        created_at datetime DEFAULT CURRENT_TIMESTAMP NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    dbDelta($sql);
}

function naver_indexnow_multisite_setup() {
    if (is_multisite()) {
        $sites = get_sites();
        foreach ($sites as $site) {
            switch_to_blog($site->blog_id);
            naver_indexnow_create_log_table();
            restore_current_blog();
        }
    }
}

add_action('wpmu_new_blog', 'naver_indexnow_create_log_table');
add_action('wp_initialize_site', 'naver_indexnow_create_log_table');

naver_indexnow_multisite_setup();
?>